﻿using UnityEngine;

namespace Oroxylum
{
    public static partial class Extensions
    {
        public static Transform GetParent(this Component compt)
        {
            if (compt == null)
                return null;
            return compt.transform.parent;
        }

        public static Transform GetParent(this GameObject obj)
        {
            if (obj == null)
                return null;
            return obj.transform.parent;
        }

        public static void SetParent(this Component compt, GameObject target)
        {
            if (compt == null)
                return;
            if (target == null)
                compt.transform.SetParent(null);
            compt.transform.SetParent(target.transform);
        }


        public static void SetParent(this GameObject obj, Component target, bool resetTrans = false)
        {
            if (obj == null || target == null)
                return;

            obj.transform.SetParent(target.transform);
            if (resetTrans)
            {
                obj.transform.localPosition = Vector3.zero;
                obj.transform.localRotation = Quaternion.identity;
            }
        }

        public static void SetParent(this GameObject obj, GameObject target, bool resetTrans = false) { obj?.SetParent(target.transform, resetTrans); }

        public static Vector3 GetEulerAngle(this Component compT)
        {
            if (compT == null)
                return Vector3.zero;

            return compT.transform.eulerAngles;
        }

        public static void SetLocalScale(this Component trans, Vector3 scale)
        {
            if (trans == null)
                return;

            trans.transform.localScale = scale;
        }

        public static void SetSortingOrder(this Canvas canvas, int targetSort)
        {
            if (canvas == null)
                return;
            canvas.sortingOrder = targetSort;
        }

        public static void SetLocalScale(this GameObject obj, Vector3 scale)
        {
            if (obj == null)
                return;

            obj.transform.localScale = scale;
        }

        public static void SetPosition(this GameObject obj, Vector3 pos)
        {
            if (obj == null)
                return;

            obj.transform.position = pos;
        }

        public static void SetLocalPosition(this GameObject obj, Vector3 pos)
        {
            if (obj == null)
                return;

            obj.transform.localPosition = pos;
        }

        public static void SetEnable(this Collider collider, bool enable)
        {
            if (collider == null)
                return;

            collider.enabled = enable;
        }

        public static void SetEnable(this Behaviour aBehaviour, bool enable)
        {
            if (aBehaviour == null)
                return;

            aBehaviour.enabled = enable;
        }


    }
}
