﻿/*
 *Copyright(C) 2023 by  Chief All rights reserved.
 *Unity版本：2022.3.10f1 
 *作者:Chief  
 *创建日期: 2023-10-20 
 *模块说明：二进制读写辅助
 *版本: 1.0.1
*/

using System.Collections.Generic;
using System.IO;
using Unity.Mathematics;
using UnityEngine.Rendering;

namespace Oroxylum
{
    /// <summary>
    /// 二进制读写辅助
    /// </summary>
    public static class BinaryHelper_Math 
    {

        public static void Write(this BinaryWriter writer, float2 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
        }

        public static float2 ReadFloat2(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            return new float2(x, y);
        }

        public static void Write(this BinaryWriter writer, float3 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
        }

        public static float3 ReadFloat3(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            float z = reader.ReadSingle();
            return new float3(x, y, z);
        }

        public static void Write(this BinaryWriter writer, float4 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
            writer.Write(value.w);
        }

        public static float4 ReadFloat4(this BinaryReader reader)
        {
            float x = reader.ReadSingle();
            float y = reader.ReadSingle();
            float z = reader.ReadSingle();
            float w = reader.ReadSingle();
            return new float4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, float4x4 value)
        {
            writer.Write(value.c0);
            writer.Write(value.c1);
            writer.Write(value.c2);
            writer.Write(value.c3);
        }

        public static float4x4 ReadFloat4x4(this BinaryReader reader)
        {
            float4 c0 = reader.ReadFloat4();
            float4 c1 = reader.ReadFloat4();
            float4 c2 = reader.ReadFloat4();
            float4 c3 = reader.ReadFloat4();
            return new float4x4(c0, c1, c2, c3);
        }

        public static void Write(this BinaryWriter writer, int2 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
        }

        public static int2 ReadInt2(this BinaryReader reader)
        {
            int x = reader.ReadInt32();
            int y = reader.ReadInt32();
            return new int2(x, y);
        }

        public static void Write(this BinaryWriter writer, int3 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
        }

        public static int3 ReadInt3(this BinaryReader reader)
        {
            int x = reader.ReadInt32();
            int y = reader.ReadInt32();
            int z = reader.ReadInt32();
            return new int3(x, y, z);
        }

        public static void Write(this BinaryWriter writer, int4 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
            writer.Write(value.w);
        }

        public static int4 ReadInt4(this BinaryReader reader)
        {
            int x = reader.ReadInt32();
            int y = reader.ReadInt32();
            int z = reader.ReadInt32();
            int w = reader.ReadInt32();
            return new int4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, quaternion value)
        {
            writer.Write(value.value);
        }

        public static quaternion ReadQuaternion(this BinaryReader reader)
        {
            var value = reader.ReadFloat4();
            return new quaternion(value);
        }

        public static void WriteHalf(this BinaryWriter writer, half value)
        {
            writer.Write(value.value);
        }

        public static half ReadHalf(this BinaryReader reader)
        {
            ushort x = reader.ReadUInt16();
            half v = half.zero;
            v.value = x;
            return v;
        }

        public static void WriteHalf(this BinaryWriter writer, half2 value)
        {
            writer.WriteHalf(value.x);
            writer.WriteHalf(value.y);
        }

        public static half2 ReadHalf2(this BinaryReader reader)
        {
            half x = reader.ReadHalf();
            half y = reader.ReadHalf();
            return new half2(x, y);
        }

        public static void WriteHalf(this BinaryWriter writer, half3 value)
        {
            writer.WriteHalf(value.x);
            writer.WriteHalf(value.y);
            writer.WriteHalf(value.z);
        }

        public static half3 ReadHalf3(this BinaryReader reader)
        {
            half x = reader.ReadHalf();
            half y = reader.ReadHalf();
            half z = reader.ReadHalf();
            return new half3(x, y, z);
        }

        public static void WriteHalf(this BinaryWriter writer, half4 value)
        {
            writer.WriteHalf(value.x);
            writer.WriteHalf(value.y);
            writer.WriteHalf(value.z);
            writer.WriteHalf(value.w);
        }

        public static half4 ReadHalf4(this BinaryReader reader)
        {
            half x = reader.ReadHalf();
            half y = reader.ReadHalf();
            half z = reader.ReadHalf();
            half w = reader.ReadHalf();
            return new half4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, ShadowCastingMode shadowCastingMode)
        {
            writer.Write((int)shadowCastingMode);
        }


        public static void WriteSafe(this BinaryWriter writer, quaternion quaternion)
        {
            writer.WriteSafe(quaternion.value);
        }

        public static quaternion ReadQuaternionSafe(this BinaryReader reader)
        {
            float4 value = reader.ReadFloat4Safe();
            return new quaternion(value);
        }

        public static void WriteSafe(this BinaryWriter writer, float4 value)
        {
            writer.WriteSafe(value.x);
            writer.WriteSafe(value.y);
            writer.WriteSafe(value.z);
            writer.WriteSafe(value.w);
        }

        public static float4 ReadFloat4Safe(this BinaryReader reader)
        {
            float x = reader.ReadFloatSafe();
            float y = reader.ReadFloatSafe();
            float z = reader.ReadFloatSafe();
            float w = reader.ReadFloatSafe();
            return new float4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, bool2 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
        }

        public static bool2 ReadBool2(this BinaryReader reader)
        {
            bool x = reader.ReadBoolean();
            bool y = reader.ReadBoolean();
            return new bool2(x, y);
        }

        public static void Write(this BinaryWriter writer, bool3 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
        }

        public static bool3 ReadBool3(this BinaryReader reader)
        {
            bool x = reader.ReadBoolean();
            bool y = reader.ReadBoolean();
            bool z = reader.ReadBoolean();
            return new bool3(x, y, z);
        }

        public static void Write(this BinaryWriter writer, bool4 value)
        {
            writer.Write(value.x);
            writer.Write(value.y);
            writer.Write(value.z);
            writer.Write(value.w);
        }

        public static bool4 ReadBool4(this BinaryReader reader)
        {
            bool x = reader.ReadBoolean();
            bool y = reader.ReadBoolean();
            bool z = reader.ReadBoolean();
            bool w = reader.ReadBoolean();
            return new bool4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, float2x3 value)
        {
            writer.Write(value.c0);
            writer.Write(value.c1);
            writer.Write(value.c2);
        }

        public static float2x3 ReadFloat2x3(this BinaryReader reader)
        {
            float2 c0 = reader.ReadFloat2();
            float2 c1 = reader.ReadFloat2();
            float2 c2 = reader.ReadFloat2();
            return new float2x3(c0, c1, c2);
        }

        public static void Write(this BinaryWriter writer, float3x3 value)
        {
            writer.Write(value.c0);
            writer.Write(value.c1);
            writer.Write(value.c2);
        }

        public static float3x3 ReadFloat3x3(this BinaryReader reader)
        {
            float3 c0 = reader.ReadFloat3();
            float3 c1 = reader.ReadFloat3();
            float3 c2 = reader.ReadFloat3();
            return new float3x3(c0, c1, c2);
        }

        public static void Write(this BinaryWriter writer, float3x4 value)
        {
            writer.Write(value.c0);
            writer.Write(value.c1);
            writer.Write(value.c2);
            writer.Write(value.c3);
        }

        public static float3x4 ReadFloat3x4(this BinaryReader reader)
        {
            float3 x = reader.ReadFloat3();
            float3 y = reader.ReadFloat3();
            float3 z = reader.ReadFloat3();
            float3 w = reader.ReadFloat3();
            return new float3x4(x, y, z, w);
        }

        public static void Write(this BinaryWriter writer, half[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.WriteHalf(array[i]);
        }

        public static half[] ReadHalfArray(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            half[] array = new half[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadHalf();
            return array;
        }

        public static void Write(this BinaryWriter writer, half4[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.WriteHalf(array[i]);
        }

        public static half4[] ReadHalf4Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            half4[] array = new half4[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadHalf4();
            return array;
        }

        public static void Write(this BinaryWriter writer, int4[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static void Write(this BinaryWriter writer, List<int4> array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Count;
            writer.Write(length);
            for (int i = 0; i < length; i++)
                writer.Write(array[i]);
        }

        public static int4[] ReadInt4Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            int4[] array = new int4[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadInt4();
            return array;
        }

        public static void Write(this BinaryWriter writer, float2[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
            {
                var value = array[i];
                writer.Write(value);
            }
        }

        public static float2[] ReadFloat2Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            float2[] array = new float2[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadFloat2();
            return array;
        }

        public static void Write(this BinaryWriter writer, float3[] array)
        {
            if (array == null)
            {
                writer.Write(0);
                return;
            }

            int length = array.Length;
            writer.Write(length);
            for (int i = 0; i < length; i++)
            {
                var value = array[i];
                writer.Write(value);
            }
        }

        public static float3[] ReadFloat3Array(this BinaryReader reader)
        {
            int length = reader.ReadInt32();
            float3[] array = new float3[length];
            for (int i = 0; i < length; i++)
                array[i] = reader.ReadFloat3();
            return array;
        }

    }
}
