﻿/*
 *Copyright(C) 2020 by  Cyf All rights reserved.
 *Unity版本：2021.3.11f1c2 
 *作者:程一峰  
 *创建日期: 2022-11-10 
 *模块说明：木蝴蝶数学库：数据格式
 *版本: 1.0
*/

using System;
using Unity.Mathematics;

namespace Oroxylum
{
    /// <summary>
    /// 2d直线
    /// </summary>
    public struct Line2d
    {
        public float2 Origin;

        /// <summary>
        /// 直线方向，已经 normalize
        /// </summary>
        public float2 Direction;

        public Line2d(float2 origin, float2 direction)
        {
            this.Origin = origin;
            this.Direction = math.normalize(direction);
        }

        public static Line2d FromPoints(float2 p0, float2 p1)
        {
            return new Line2d(p0, math.normalize(p1 - p0));
        }

        public double Project(float2 p) { return (p - Origin).Dot(Direction); }

        public float2 ClosestPoint(float2 p)
        {
            float t = (p - Origin).Dot(Direction);
            return Origin + (t * Direction);
        }

        public double DistanceSquared(float2 p)
        {
            float t = (p - Origin).Dot(Direction);
            float2 proj = Origin + (t * Direction);
            return (proj - p).LengthSquared();
        }

        /// <summary>
        /// 检测点在直线的哪一测
        /// </summary>
        /// <param name="test">待检测的点</param>
        /// <param name="tol">直线粗细宽度</param>
        /// <returns>1：右侧；-1：左侧：0 在直线上</returns>
        public int WhichSide(float2 test, float tol = 0)
        {
            float x0 = test.x - Origin.x;
            float y0 = test.y - Origin.y;
            float x1 = Direction.x;
            float y1 = Direction.y;
            float det = (x0 * y1) - (x1 * y0);
            return det > tol ? +1 : (det < -tol ? -1 : 0);
        }

        public float2 IntersectionPoint(ref Line2d other, float dotThresh = OMath.ZeroTolerance)
        {
            // see IntrLine2Line2 for explanation of algorithm
            float2 diff = other.Origin - Origin;
            float D0DotPerpD1 = Direction.DotPerp(other.Direction);
            if (Math.Abs(D0DotPerpD1) > dotThresh)
            {                    // Lines intersect in a single point.
                float invD0DotPerpD1 = 1 / D0DotPerpD1;
                float diffDotPerpD1 = diff.DotPerp(other.Direction);
                float s = diffDotPerpD1 * invD0DotPerpD1;
                return Origin + (s * Direction);
            }
            // Lines are parallel.
            return float.MaxValue;
        }

    }
}