﻿/*
 *Copyright(C) 2020 by  Cyf All rights reserved.
 *Unity版本：2021.3.11f1c2 
 *作者:程一峰  
 *创建日期: 2022-11-10 
 *模块说明：木蝴蝶数学库：数据格式
 *版本: 1.0
*/

using Unity.Mathematics;
using UnityEngine;

namespace Oroxylum
{
    /// <summary>
    /// 3d 直线
    /// </summary>
    public struct Line3d
    {
        public float3 Origin;

        /// <summary>
        /// 直线方向，已经 normalize
        /// </summary>
        public float3 Direction;

        public Line3d(float3 origin, float3 direction)
        {
            this.Origin = origin;
            this.Direction = math.normalize(direction);
        }
        public Line3d(Ray ray)
        {
            this.Origin = ray.origin;
            this.Direction = ray.direction;
        }

        public static Line3d FromPoints(float3 p0, float3 p1)
        {
            return new Line3d(p0, math.normalize(p1 - p0));
        }

        public float Project(float3 p) { return (p - Origin).Dot(Direction); }

        public float3 ClosestPoint(float3 p)
        {
            float t = (p - Origin).Dot(Direction);
            return Origin + t * Direction;
        }

        public float DistanceSquared(float3 p)
        {
            float t = (p - Origin).Dot(Direction);
            float3 proj = Origin + t * Direction;
            return (proj - p).LengthSquared();
        }

    }
}